import base64
import json
import os
import re
import urllib.request

def makeRequest(url, data, apiKey):
    req =  urllib.request.Request(url, data=bytes(data, encoding='utf-8'))
    req.add_header('User-Agent', 'Mozilla/5.0')
    req.add_header('Content-Type', 'application/json')
    req.add_header('Authorization', apiKey)
    try:
        res = urllib.request.urlopen(req)
    except urllib.error.HTTPError as e:
        return e.read()

    return res.read()

def checkIfValidURL(url):
    regex = re.compile(
            r'^(?:http|ftp)s?://' # http:// or https://
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|' #domain...
            r'localhost|' #localhost...
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})' # ...or ip
            r'(?::\d+)?' # optional port
            r'(?:/?|[/?]\S+)$', re.IGNORECASE)

    return (re.match(regex, url) is not None)

def isBase64(sb):
    try:
        if isinstance(sb, str):
            # If there's any unicode here, an exception will be thrown and the function will return false
            sb_bytes = bytes(sb, 'ascii')
        elif isinstance(sb, bytes):
            sb_bytes = sb
        else:
            raise ValueError("Argument must be string or bytes")
        return base64.b64encode(base64.b64decode(sb_bytes)) == sb_bytes
    except Exception:
        return False


from __future__ import absolute_import, division, print_function

import os
import re
import base64
import json
import urllib
from .tags import _runTags
from .captions import _runCaptions
from .documents import _runDocuments, _runTextAnalysis

# SmartLens Python bindings
# API docs at https://smartlens.ai/docs
# Authors:
# Michael Royzen <michael@smartlens.ai>

# Configuration variables

api_key = None

def runTags(image):
    response = _runTags(image, api_key)
    return response

def runCaptions(image):
    response = _runCaptions(image, api_key)
    return response

def runDocumentAI(document):
    response = _runDocuments(document, api_key)
    return response

def runTextAnalysis(text, customExtractions=None):
    response = _runTextAnalysis(text, customExtractions, api_key)
    return response


import os
import re
import urllib
import base64
import json
from .util import makeRequest, checkIfValidURL, isBase64

def _runCaptions(image, apiKey):
    imageBytes = None

    if (os.path.exists(image)):
        # Open a local image
        imageBytes = open(image, 'rb').read()
        imageBytes = base64.b64encode(imageBytes).decode('utf-8')
    elif (isBase64(image)):
        if isinstance(image, str):
            imageBytes = image
        elif isinstance(image, bytes):
            imageBytes = str(image, encoding='utf-8')
    elif (checkIfValidURL(image)):
        try:
            response = urllib.request.urlopen(image)
            imageBytes = response.read()
        except urllib.error.HTTPError:
            return {"error": "The URL you passed in is invalid."}
        imageBytes = base64.b64encode(imageBytes).decode('utf-8')
    else:
        return {"error": "Your \"image\" parameter needs to be either a path to a local image, a string of base64 bytes, or a URL to a remote image."}

    # Compose a JSON Predict request
    data = json.dumps({'image': imageBytes})

    url = 'https://api.smartlens.ai/v1/models/caption/predict'
    r = makeRequest(url, data, apiKey)
    r = r.decode('utf-8')

    dictResponse = json.loads(r)

    return dictResponse
